﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;

namespace TestPalette
{
    public class TextBoxHelper
    {
        private static readonly Regex _regex = new Regex("[^0-9.-]+");

        #region OnlyNumeric              
        public static bool GetOnlyNumeric(DependencyObject obj) => (bool)obj.GetValue(OnlyNumericProperty);
        public static void SetOnlyNumeric(DependencyObject obj, bool value) => obj.SetValue(OnlyNumericProperty, value);
        public static readonly DependencyProperty OnlyNumericProperty =
            DependencyProperty.RegisterAttached("OnlyNumeric",
                typeof(bool),
                typeof(TextBoxHelper),
                new PropertyMetadata(false
                , (ss, ee) =>
                {
                    if (ss is TextBoxBase tb)
                    {
                        tb.PreviewTextInput += (sss, eee) =>
                        {
                            eee.Handled = _regex.IsMatch(eee.Text);
                        };
                    }
                }));
        #endregion

        #region ClosePopupOnEnter              
        public static bool GetClosePopupOnEnter(DependencyObject obj) => (bool)obj.GetValue(ClosePopupOnEnterProperty);
        public static void SetClosePopupOnEnter(DependencyObject obj, bool value) => obj.SetValue(ClosePopupOnEnterProperty, value);
        public static readonly DependencyProperty ClosePopupOnEnterProperty =
            DependencyProperty.RegisterAttached("ClosePopupOnEnter",
                typeof(bool),
                typeof(TextBoxHelper),
                new PropertyMetadata(false
                , (ss, ee) =>
                {
                    if (ss is TextBoxBase control)
                    {
                        control.KeyUp += (ss, ee) =>
                        {
                            if (ee.Key == System.Windows.Input.Key.Enter)
                            {
                                var b = GetVisualParent<Border>(control);
                                var p = GetLogicalParent<Popup>(b);
                                p.IsOpen = false;
                            }
                        };
                    }
                }));
        #endregion

        #region ClosePopupOnClick              
        public static bool GetClosePopupOnClick(DependencyObject obj) => (bool)obj.GetValue(ClosePopupOnClickProperty);
        public static void SetClosePopupOnClick(DependencyObject obj, bool value) => obj.SetValue(ClosePopupOnClickProperty, value);
        public static readonly DependencyProperty ClosePopupOnClickProperty =
            DependencyProperty.RegisterAttached("ClosePopupOnClick",
                typeof(bool),
                typeof(TextBoxHelper),
                new PropertyMetadata(false
                , (ss, ee) =>
                {
                    if (ss is UIElement control)
                    {
                        control.PreviewMouseLeftButtonUp += delegate
                        {
                            var b = GetVisualParent<Border>(control);
                            var p = GetLogicalParent<Popup>(b);
                            p.IsOpen = false;
                        };
                    }
                }));
        #endregion

        #region SelectAllOnFocus              
        public static bool GetSelectAllOnFocus(DependencyObject obj) => (bool)obj.GetValue(SelectAllOnFocusProperty);
        public static void SetSelectAllOnFocus(DependencyObject obj, bool value) => obj.SetValue(SelectAllOnFocusProperty, value);
        public static readonly DependencyProperty SelectAllOnFocusProperty =
            DependencyProperty.RegisterAttached("SelectAllOnFocus",
                typeof(bool),
                typeof(TextBoxHelper),
                new PropertyMetadata(false
                , (ss, ee) =>
                {
                    if (ss is TextBox control)
                    {
                        control.GotFocus += delegate
                        {
                            control.SelectAll();
                        };
                    }
                }));
        #endregion

        #region ValidateOnEnter
        public static bool GetValidateOnEnter(DependencyObject obj) => (bool)obj.GetValue(ValidateOnEnterProperty);
        public static void SetValidateOnEnter(DependencyObject obj, bool value) => obj.SetValue(ValidateOnEnterProperty, value);
        public static readonly DependencyProperty ValidateOnEnterProperty =
            DependencyProperty.RegisterAttached(
                "ValidateOnEnter",
                typeof(bool),
                typeof(TextBoxHelper),
                new FrameworkPropertyMetadata(false, (d, e) =>
                        {
                            if (d is TextBox control)
                            {
                                control.KeyUp += (ss, ee) =>
                                {
                                    if (ee.Key == System.Windows.Input.Key.Enter)
                                    {
                                        var binding = control.GetBindingExpression(TextBox.TextProperty);
                                        if (binding != null) binding.UpdateSource();
                                        control.SelectAll();
                                    }
                                };
                            }
                        }));
        #endregion

        #region STATIC 
        public static T GetVisualParent<T>(DependencyObject child) where T : FrameworkElement
        {
            while ((child != null) && !(child is T))
            {
                child = VisualTreeHelper.GetParent(child);
            }
            return child as T;
        }
        public static T GetLogicalParent<T>(DependencyObject child) where T : FrameworkElement
        {
            while ((child != null) && !(child is T))
            {
                child = LogicalTreeHelper.GetParent(child);
            }
            return child as T;
        }
        #endregion
    }
}
