﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace UltimateCorp.Palette
{

    public delegate void EndAnimationEventHandler();

    //[TemplatePart("PART_BaseLine", typeof(Line))]
    //[TemplatePart("PART_Extent", typeof(Line))]
    [TemplatePart(Name = "PART_Canvas", Type = typeof(Canvas))]
    public class BasePaletteControl : ContentControl
    {
        private const double MINIMUM_SIZE = 50D;

        #region Constructors
        static BasePaletteControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(BasePaletteControl), 
                new FrameworkPropertyMetadata(typeof(BasePaletteControl)));
        }

        public BasePaletteControl() { }
        #endregion       

        #region DP

        #region ShowLine (DP SHORT)
        [Category("Palette")]
        public bool ShowLine { get { return (bool)GetValue(ShowLineProperty); } set { SetValue(ShowLineProperty, value); } }
        public static readonly DependencyProperty ShowLineProperty = DependencyProperty.Register("ShowLine", typeof(bool), typeof(BasePaletteControl), new PropertyMetadata(false));
        #endregion

        #region LineColor (DP SHORT)
        [Category("Palette")]
        public Brush LineColor { get { return (Brush)GetValue(LineColorProperty); } set { SetValue(LineColorProperty, value); } }
        public static readonly DependencyProperty LineColorProperty = DependencyProperty.Register("LineColor", typeof(Brush), typeof(BasePaletteControl), new PropertyMetadata(Brushes.White));
        #endregion

        #region ContentLine (DP SHORT)
        [Browsable(false)]
        public UIElement ContentLine { get { return (UIElement)GetValue(ContentLineProperty); } set { SetValue(ContentLineProperty, value); } }
        public static readonly DependencyProperty ContentLineProperty = DependencyProperty.Register("ContentLine", typeof(UIElement), typeof(BasePaletteControl), new PropertyMetadata(null));
        #endregion

        #region AdjustTop (DP SHORT)
        [Category("Palette")]
        public double AdjustTop { get { return (double)GetValue(AdjustTopProperty); } set { SetValue(AdjustTopProperty, value); } }
        public static readonly DependencyProperty AdjustTopProperty = 
            DependencyProperty.Register("AdjustTop", 
                typeof(double), typeof(BasePaletteControl), 
                new PropertyMetadata(0D, UpdateSizeStatic));
        #endregion

        #region CornerRadius (DP SHORT)
        [Category("Palette")]
        public CornerRadius CornerRadius { get { return (CornerRadius)GetValue(CornerRadiusProperty); } set { SetValue(CornerRadiusProperty, value); } }
        public static readonly DependencyProperty CornerRadiusProperty = 
            DependencyProperty.Register("CornerRadius", 
                typeof(CornerRadius), typeof(BasePaletteControl), 
                new PropertyMetadata(new CornerRadius(2.0)));
        #endregion

        #endregion

        #region ApplyTemplate
        Canvas canvas = null;
        ContentPresenter content = null;        
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            canvas = GetTemplateChild("PART_Canvas") as Canvas;
            content = GetTemplateChild("PART_Content") as ContentPresenter;            
            content.SizeChanged += delegate
            {
                UpdateSize();
            };
            UpdateSize();
        }
        #endregion

        #region Size
        private static void UpdateSizeStatic(object sender,
            DependencyPropertyChangedEventArgs args = new DependencyPropertyChangedEventArgs()) =>
               (sender as BasePaletteControl).UpdateSize();

        protected virtual void UpdateSize()
        {
            if (canvas != null)
            {
                try
                {
                    double w = content.ActualWidth;                    
                    double h = content.ActualHeight;
                    double a = AdjustTop;
                    if (content.Content == null)
                    {
                        if (w < MINIMUM_SIZE) w = MINIMUM_SIZE;
                        if (h < MINIMUM_SIZE) h = MINIMUM_SIZE;
                        a = 0;
                    }
                    canvas.Width = w;
                    canvas.Height = h + a;
                    if (double.IsNaN(this.Width)) MinWidth = canvas.Width;
                    else MinWidth = 0D;
                    if(double.IsNaN(this.Height)) MinHeight = canvas.Height;
                    else MinHeight = 0D;
                    content.Margin = new Thickness(0, a, 0, 0);
                }
                catch { }
            }
        }

        #endregion       
    }
}
