﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;

namespace UltimateCorp.Palette
{
    public class PaletteArrayControl : PaletteStringControl
    {
        #region Pause (DP SHORT)
        public int Pause { get { return (int)GetValue(PauseProperty); } set { SetValue(PauseProperty, value); } }
        public static readonly DependencyProperty PauseProperty = DependencyProperty.Register("Pause", typeof(int), typeof(PaletteArrayControl), new PropertyMetadata(1000));
        #endregion

        #region Array
        public object Array { get => (object)GetValue(ArrayProperty); set => SetValue(ArrayProperty, value); }
        public static readonly DependencyProperty ArrayProperty =
            DependencyProperty.Register(
                "Array",
                typeof(object),
                typeof(PaletteArrayControl),
                new PropertyMetadata(null
                , (ss, ee) =>
                {
                    var pac = ss as PaletteArrayControl;
                    if (ee.NewValue == null)
                    {
                        pac.ClearText();
                    }
                    else
                    {
                        Type valueType = ee.NewValue.GetType();
                        if (!valueType.IsArray ||
                            !valueType.GetElementType().Equals(typeof(string)))
                        {
                            pac.ClearText();
                        }
                        else
                        {
                            if (!pac.IsInitialized || DesignerProperties.GetIsInDesignMode(pac))
                            {
                                pac.SetText();
                            }
                            else
                            {
                                pac.StartText();
                            }
                        }
                    }
                }));
        #endregion       

        private int pos = -1;
        private bool onAnimation = false;
        private void SetText()
        {
            var ss = (string[])Array;
            Text = ss[++pos];
        }

        private void StartText()
        {
            pos = -1;
            var ss = (string[])Array;
            EndAnimationEventHandler d = null;
            d = () =>
            {
                if (pos + 1 >= ss.Length)
                {
                    CompletAnim -= d;
                }
                else
                {
                    Thread.Sleep(Pause);
                    SetText();
                }
            };
            CompletAnim += d;
            SetText();
        }

        private void ClearText()
        {
            pos = -1;
            Text = null;
        }
    }
}
