﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace UltimateCorp.Palette
{
    public class PaletteCharControl : BasePaletteDefinition
    {
        protected override string DefinitionNoCase => CaseSensitive ? AllowedChars : AllowedChars.ToLower();
        protected override string DefinitionCase => AllowedChars;

        protected string Convert(string s) => CaseSensitive ? s : s.ToLower();

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            if (this.Char == null)
            {
                this.Text = DefinitionCase[0].ToString();
            }
        }

        #region AllowedChars (DP SHORT)
        [Category("Palette")]
        public string AllowedChars { get { return (string)GetValue(AllowedCharsProperty); } set { SetValue(AllowedCharsProperty, value); } }
        public static readonly DependencyProperty AllowedCharsProperty = DependencyProperty.Register(
            "AllowedChars",
            typeof(string),
            typeof(PaletteCharControl),
            new PropertyMetadata(" 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"));
        #endregion

        #region CaseSensitive (DP SHORT)
        [Category("Palette")]
        public bool CaseSensitive { get { return (bool)GetValue(CaseSensitiveProperty); } set { SetValue(CaseSensitiveProperty, value); } }
        public static readonly DependencyProperty CaseSensitiveProperty = DependencyProperty.Register("CaseSensitive", typeof(bool), typeof(PaletteCharControl), new PropertyMetadata(false));
        #endregion

        #region Char
        [Category("Palette")]
        public string Char { get => (string)GetValue(CharProperty); set => SetValue(CharProperty, value); }
        public static readonly DependencyProperty CharProperty = DependencyProperty.Register(
            "Char",
            typeof(string),
            typeof(PaletteCharControl),
            new PropertyMetadata(null
            , (ss, ee) =>
            {
                if (ss is PaletteCharControl pnc)
                {
                    if (!pnc.IsInitialized ||
                        DesignerProperties.GetIsInDesignMode(pnc))
                    {
                        int pos = 0;
                        if (ee.NewValue != null)
                        {
                            var s = pnc.Convert(ee.NewValue.ToString());
                            pos = pnc.DefinitionNoCase.IndexOf(s);
                        }
                        pnc.DisableAnim = true;
                        pnc.Text = pnc.DefinitionCase[pos].ToString();
                        pnc.DisableAnim = false;
                        pnc.SetAll();
                    }
                    else
                    {
                        string from = pnc.DefinitionCase[0].ToString();
                        if (ee.OldValue != null)
                        {
                            from = pnc.Convert(ee.OldValue.ToString());
                        }
                        string to = pnc.Convert(ee.NewValue.ToString());
                        if (!from.Equals(to)) pnc.MoveTo(from, to);
                    }
                }
            },
            (ss, ee) =>
            {
                if (ee == null) return null;
                var pcc = ss as PaletteCharControl;
                string s = pcc.DefinitionCase[0].ToString();
                try
                {
                    s = pcc.Convert(((string)ee)[0].ToString());
                    if (pcc.DefinitionNoCase.IndexOf(s) == -1)
                        s = pcc.DefinitionCase[0].ToString();
                }
                catch { }
                return s;
            }));
        #endregion
    }
}
