﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace UltimateCorp.Palette
{
    [DefaultProperty("Content")]
    [ContentProperty("Content")]
    public class PaletteContentControl : BasePaletteAnimation
    {
        #region Content (DP SHORT)
        [Browsable(false)]
        public object Content { get { return (object)GetValue(ContentProperty); } set { SetValue(ContentProperty, value); } }
        public static readonly DependencyProperty ContentProperty =
            DependencyProperty.Register("Content", typeof(object), typeof(PaletteContentControl),
                new PropertyMetadata(null, (ss, ee) =>
                {
                    if (ss is PaletteContentControl pc)
                    {
                        pc.UpdateSize();
                        if (ee.OldValue == null) { }
                        else
                        {
                            if (DesignerProperties.GetIsInDesignMode(pc))
                            {
                                pc.paletteTop.Content = pc.Content;
                                pc.paletteBottom.Content = pc.Content;
                            }
                            else
                                pc.StartAnimation(ee.OldValue as UIElement);
                        }
                    }
                }));
        #endregion

        protected override UIElement GetContent => (Content as UIElement).CloneElement();
        protected override bool HasContent => Content != null;

        #region ApplyTemplate       
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();           
            UpdateSize();
        }
        #endregion

        private void UpdateSize()
        {
            if (Content is FrameworkElement ui)
            {
                this.MinWidth = ui.ActualWidth;
                this.MinHeight = ui.ActualHeight;
            }
            else
            {
                this.MinWidth = this.MinHeight = MINIMUM_SIZE;
            }
        }
    }
}

