﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;

namespace UltimateCorp.Palette
{
    public class PaletteStringControl : Control, INotifyPropertyChanged
    {
        #region Event
        public event EndAnimationEventHandler CompletAnim;
        public event PropertyChangedEventHandler? PropertyChanged;
        private void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        #region Constructor & ApplyTemplate
        static PaletteStringControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(PaletteStringControl),
                new FrameworkPropertyMetadata(typeof(PaletteStringControl)));
        }

        StackPanel panel = null;
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            panel = GetTemplateChild("PART_Content") as StackPanel;
            UpdateChars();
        }
        #endregion

        List<PaletteCharControl> pcc = new List<PaletteCharControl>();
        private void UpdateChars()
        {
            if (panel != null)
            {
                panel.Children.Clear();
                pcc.Clear();
                for (int i = 0; i < CharCount; i++)
                {
                    // add separator
                    if (i != 0)
                    {
                        var t = new TextBlock();
                        t.VerticalAlignment = VerticalAlignment.Center;
                        t.HorizontalAlignment = HorizontalAlignment.Left;
                        t.Height = 2;
                        SetBinding(this, t, "CharMargin", TextBlock.WidthProperty);
                        panel.Children.Add(t);
                    }
                    //
                    {
                        var p = new PaletteCharControl();                        
                        p.Name = $"P{i:D2}";
                        p.Text = AllowedChars[0].ToString();
                        SetBinding(this, p, "AnimationTime", PaletteCharControl.AnimationTimeProperty);
                        SetBinding(this, p, "AdjustTop", PaletteCharControl.AdjustTopProperty);
                        SetBinding(this, p, "ShowLine", PaletteCharControl.ShowLineProperty);
                        SetBinding(this, p, "LineColor", PaletteCharControl.LineColorProperty);
                        SetBinding(this, p, "ContentLine", PaletteCharControl.ContentLineProperty);
                        SetBinding(this, p, "CharBackground", PaletteCharControl.BackgroundProperty);
                        SetBinding(this, p, "CharCornerRadius", PaletteCharControl.CornerRadiusProperty);
                        SetBinding(this, p, "CharBorderBrush", PaletteCharControl.BorderBrushProperty);
                        SetBinding(this, p, "CharBorderThickness", PaletteCharControl.BorderThicknessProperty);
                        SetBinding(this, p, "CharWidth", PaletteCharControl.WidthProperty);
                        SetBinding(this, p, "CharHeight", PaletteCharControl.HeightProperty);
                        SetBinding(this, p, "AllowedChars", PaletteCharControl.AllowedCharsProperty);
                        SetBinding(this, p, "CaseSensitive", PaletteCharControl.CaseSensitiveProperty);
                        SetBinding(this, p, "Foreground", PaletteCharControl.ForegroundProperty);
                        pcc.Add(p);
                        panel.Children.Add(p);
                        p.CompletAnim += CheckEndAnim;
                    }
                }
            }
            UpdateText(true);
        }

        public void UpdateBrushes(Brush[] brushes)
        {
            int i = 0;
            foreach (var item in pcc)
            {
                if (brushes == null)
                {
                    SetBinding(this, item, "Foreground", PaletteCharControl.ForegroundProperty);
                }
                else
                {
                    item.Foreground = brushes[i];
                }
                i++;
            }
        }

        private void SetBinding(object source, FrameworkElement destination, string property, DependencyProperty dp)
        {
            Binding binding = new Binding(property);
            binding.Source = source;
            destination.SetBinding(dp, binding);
        }

        private void UpdateText(bool init = false)
        {
            if (pcc.Count == 0) return;
            OnAnimated = true;
            var t = Text + new string(' ', this.CharCount + 10);
            t = t.Substring(0, CharCount);
            //Debug.WriteLine($"**{t}*");
            int i = 0;
            foreach (var item in t)
            {
                if (DesignerProperties.GetIsInDesignMode(this) || init)
                {
                    pcc[i].Text = item.ToString();
                    // pcc[i].Char = item.ToString();
                    OnAnimated = false;
                }
                else
                {
                    pcc[i].Char = item.ToString();
                }
                i++;
            }
        }

        #region OnAnimated
        private bool _OnAnimated = false;
        public bool OnAnimated
        {
            get => _OnAnimated;
            private set
            {
                _OnAnimated = value;
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("OnAnimated"));
            }
        }
        #endregion

        private void CheckEndAnim()
        {
            //Debug.WriteLine($"> {Info()}");
            foreach (var item in pcc)
            {
                if (item.OnAnim) return;
            }
            OnAnimated = false;
            CompletAnim?.Invoke();
        }

        public string Info() => pcc.Select(x => x.OnAnim ? "1" : "0").Aggregate((a, b) => a + b);

        #region DP (for char palette control)

        #region AnimationTime (DP SHORT)
        [Category("Palette")]
        public double AnimationTime { get { return (double)GetValue(AnimationTimeProperty); } set { SetValue(AnimationTimeProperty, value); } }
        public static readonly DependencyProperty AnimationTimeProperty = DependencyProperty.Register("AnimationTime", typeof(double), typeof(PaletteStringControl), new PropertyMetadata(300D));
        #endregion

        #region AdjustTop (DP SHORT)
        [Category("Palette")]
        public double AdjustTop { get { return (double)GetValue(AdjustTopProperty); } set { SetValue(AdjustTopProperty, value); } }
        public static readonly DependencyProperty AdjustTopProperty = DependencyProperty.Register("AdjustTop", typeof(double), typeof(PaletteStringControl), new PropertyMetadata(0D));
        #endregion

        #region ShowLine (DP SHORT)
        [Category("Palette")]
        public bool ShowLine { get { return (bool)GetValue(ShowLineProperty); } set { SetValue(ShowLineProperty, value); } }
        public static readonly DependencyProperty ShowLineProperty = DependencyProperty.Register("ShowLine", typeof(bool), typeof(PaletteStringControl), new PropertyMetadata(true));
        #endregion

        #region LineColor (DP SHORT)
        [Category("Palette")]
        public Brush LineColor { get { return (Brush)GetValue(LineColorProperty); } set { SetValue(LineColorProperty, value); } }
        public static readonly DependencyProperty LineColorProperty = DependencyProperty.Register("LineColor", typeof(Brush), typeof(PaletteStringControl), new PropertyMetadata(Brushes.White));
        #endregion

        #region ContentLine (DP SHORT)
        [Browsable(false)]
        public UIElement ContentLine { get { return (UIElement)GetValue(ContentLineProperty); } set { SetValue(ContentLineProperty, value); } }
        public static readonly DependencyProperty ContentLineProperty = DependencyProperty.Register("ContentLine", typeof(UIElement), typeof(PaletteStringControl), new PropertyMetadata(null));
        #endregion

        #region CharBackground (DP SHORT)
        [Category("Palette")]
        public Brush CharBackground { get { return (Brush)GetValue(CharBackgroundProperty); } set { SetValue(CharBackgroundProperty, value); } }
        public static readonly DependencyProperty CharBackgroundProperty = DependencyProperty.Register("CharBackground", typeof(Brush), typeof(PaletteStringControl), new PropertyMetadata(Brushes.Black));
        #endregion

        #region CharCornerRadius (DP SHORT)
        [Category("Palette")]
        public CornerRadius CharCornerRadius { get { return (CornerRadius)GetValue(CharCornerRadiusProperty); } set { SetValue(CharCornerRadiusProperty, value); } }
        public static readonly DependencyProperty CharCornerRadiusProperty = DependencyProperty.Register("CharCornerRadius", typeof(CornerRadius), typeof(PaletteStringControl), new PropertyMetadata(null));
        #endregion

        #region CharBorderBrush (DP SHORT)
        [Category("Palette")]
        public Brush CharBorderBrush { get { return (Brush)GetValue(CharBorderBrushProperty); } set { SetValue(CharBorderBrushProperty, value); } }
        public static readonly DependencyProperty CharBorderBrushProperty = DependencyProperty.Register("CharBorderBrush", typeof(Brush), typeof(PaletteStringControl), new PropertyMetadata(null));
        #endregion

        #region CharBorderThickness (DP SHORT)
        [Category("Palette")]
        public Thickness CharBorderThickness { get { return (Thickness)GetValue(CharBorderThicknessProperty); } set { SetValue(CharBorderThicknessProperty, value); } }
        public static readonly DependencyProperty CharBorderThicknessProperty = DependencyProperty.Register("CharBorderThickness", typeof(Thickness), typeof(PaletteStringControl), new PropertyMetadata(new Thickness()));
        #endregion

        #region CharWidth (DP SHORT)
        [Category("Palette")]
        public double CharWidth { get { return (double)GetValue(CharWidthProperty); } set { SetValue(CharWidthProperty, value); } }
        public static readonly DependencyProperty CharWidthProperty =
            DependencyProperty.Register("CharWidth", typeof(double),
                typeof(PaletteStringControl),
                new PropertyMetadata(double.NaN,
                    (ss, ee) => { },
                    (ss, ee) =>
                    {
                        var d = (double)ee;
                        if (d < 0) d = 0;
                        return d;
                    }));
        #endregion

        #region CharHeight (DP SHORT)
        [Category("Palette")]
        public double CharHeight { get { return (double)GetValue(CharHeightProperty); } set { SetValue(CharHeightProperty, value); } }
        public static readonly DependencyProperty CharHeightProperty =
            DependencyProperty.Register("CharHeight", typeof(double),
                typeof(PaletteStringControl),
                new PropertyMetadata(double.NaN,
                    (ss, ee) => { },
                    (ss, ee) =>
                    {
                        var d = (double)ee;
                        if (d < 0) d = 0;
                        return d;
                    }));
        #endregion

        #endregion

        #region DP (for control)

        #region CharMargin
        [Category("Palette")]
        public double CharMargin { get => (double)GetValue(CharMarginProperty); set => SetValue(CharMarginProperty, value); }
        public static readonly DependencyProperty CharMarginProperty =
            DependencyProperty.Register("CharMargin",
                                        typeof(double),
                                        typeof(PaletteStringControl),
                                        new PropertyMetadata(0D,
                    (ss, ee) => { },
                    (ss, ee) =>
                    {
                        var d = (double)ee;
                        if (d < 0) d = 0;
                        return d;
                    }));
        #endregion

        #region CornerRadius (DP SHORT)
        [Category("Palette")]
        public CornerRadius CornerRadius { get { return (CornerRadius)GetValue(CornerRadiusProperty); } set { SetValue(CornerRadiusProperty, value); } }
        public static readonly DependencyProperty CornerRadiusProperty =
            DependencyProperty.Register("CornerRadius",
                typeof(CornerRadius), typeof(PaletteStringControl),
                new PropertyMetadata(new CornerRadius(2.0)));
        #endregion

        #region CharCount
        [Category("Palette")]
        public int CharCount { get => (int)GetValue(CharCountProperty); set => SetValue(CharCountProperty, value); }
        public static readonly DependencyProperty CharCountProperty =
            DependencyProperty.Register("CharCount",
                                        typeof(int),
                                        typeof(PaletteStringControl),
                                        new PropertyMetadata(10
                                        , (ss, ee) => (ss as PaletteStringControl).UpdateChars()
                                        , (ss, ee) =>
                                         {
                                             int i = (int)ee;
                                             if (i <= 0) i = 1;
                                             return i;
                                         }));
        #endregion

        #region AllowedChars (DP SHORT)
        [Category("Palette")]
        public string AllowedChars { get { return (string)GetValue(AllowedCharsProperty); } set { SetValue(AllowedCharsProperty, value); } }
        public static readonly DependencyProperty AllowedCharsProperty = DependencyProperty.Register(
            "AllowedChars",
            typeof(string),
            typeof(PaletteStringControl),
            new PropertyMetadata(" 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"));
        #endregion

        #region CaseSensitive (DP SHORT)
        [Category("Palette")]
        public bool CaseSensitive { get { return (bool)GetValue(CaseSensitiveProperty); } set { SetValue(CaseSensitiveProperty, value); } }
        public static readonly DependencyProperty CaseSensitiveProperty = DependencyProperty.Register("CaseSensitive", typeof(bool), typeof(PaletteStringControl), new PropertyMetadata(false));
        #endregion

        #region Text
        public string Text { get => (string)GetValue(TextProperty); set => SetValue(TextProperty, value); }
        public static readonly DependencyProperty TextProperty =
            DependencyProperty.Register("Text",
                                        typeof(string),
                                        typeof(PaletteStringControl),
                                        new PropertyMetadata(null
                                        , (ss, ee) => (ss as PaletteStringControl).UpdateText()
                                        ));
        #endregion

        #region ArrayBrush
        public Brush[] ArrayBrush { get => (Brush[])GetValue(ArrayBrushProperty); set => SetValue(ArrayBrushProperty, value); }
        public static readonly DependencyProperty ArrayBrushProperty =
            DependencyProperty.Register(
                "ArrayBrush",
                typeof(Brush[]),
                typeof(PaletteStringControl),
                new PropertyMetadata(null
                , (ss, ee) =>
                {
                    var psc = ss as PaletteStringControl;
                    if (ee.NewValue == null) { }
                    else
                    {
                        Type valueType = ee.NewValue.GetType();                       
                        if (valueType.IsArray &&
                            valueType.GetElementType().Equals(typeof(Brush)))
                        {
                            psc.UpdateBrushes((Brush[])ee.NewValue);
                            return;
                        }
                    }
                    psc.UpdateBrushes(null);
                }));
        #endregion

        #endregion
    }
}
