﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace UltimateCorp.Palette
{
    public delegate void SecondTickEventHandler();

    public enum NumberMode
    {
        One = 2,
        Two = 1
    }

    public class PaletteTimeControl : Control, INotifyPropertyChanged
    {
        #region Events
        public event SecondTickEventHandler SecondTick;
        public event PropertyChangedEventHandler? PropertyChanged;
        private void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #region SecondTickEvent (ROUTED EVENT)
        public event RoutedEventHandler SecondTickEvent
        {
            add => AddHandler(PaletteTimeControl.SecondTickEventEvent, value);
            remove => RemoveHandler(PaletteTimeControl.SecondTickEventEvent, value);
        }
        public static readonly RoutedEvent SecondTickEventEvent = EventManager.RegisterRoutedEvent(
            "SecondTickEvent",
            RoutingStrategy.Bubble,
            typeof(RoutedEventHandler),
            typeof(PaletteTimeControl));
        protected void OnSecondTickEvent() => this.RaiseEvent(new RoutedEventArgs(PaletteTimeControl.SecondTickEventEvent));
        #endregion

        #endregion

        #region Constructor & ApplyTemplate
        static PaletteTimeControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(PaletteTimeControl),
                new FrameworkPropertyMetadata(typeof(PaletteTimeControl)));
        }

        StackPanel panel = null;
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            panel = GetTemplateChild("PART_Element") as StackPanel;
            //
            UpdateTime();
            if (!DesignerProperties.GetIsInDesignMode(this))
            {
                DispatcherTimer dp = new DispatcherTimer(
                TimeSpan.FromMilliseconds(100),
                DispatcherPriority.Send,
                (ss, ee) =>
                {
                    UpdateTime();
                },
                this.Dispatcher);
            }
        }
        #endregion

        private string lastTime = DateTime.Now.ToLongTimeString();
           
        private void UpdateTime(bool isRun = true)
        {
            var nt = DateTime.Now.ToLongTimeString();
            if ((DesignerProperties.GetIsInDesignMode(this) ||
                isRun) && 
                    !IsPaused && (!lastTime.Equals(nt) || !IsInitialized))
            {
                lastTime = nt;  
                var dt = DateTime.Now;
                if (NumberMode == NumberMode.Two)
                {
                    H0 = dt.Hour.ToString("D2");
                    M0 = dt.Minute.ToString("D2");
                    S0 = dt.Second.ToString("D2");
                }
                else
                {
                    H0 = (dt.Hour / 10).ToString("D");
                    H1 = (dt.Hour % 10).ToString("D");
                    M0 = (dt.Minute / 10).ToString("D");
                    M1 = (dt.Minute % 10).ToString("D");
                    S0 = (dt.Second / 10).ToString("D");
                    S1 = (dt.Second % 10).ToString("D");
                }
                SecondTick?.Invoke();
                OnSecondTickEvent();
            }
        }

        #region Time properties

        #region H0
        private string _H0 = null;
        [Browsable(false)]
        public string H0 { get => _H0; set { _H0 = value; OnPropertyChanged(); } }
        #endregion
        #region H1
        private string _H1 = null;
        [Browsable(false)]
        public string H1 { get => _H1; set { _H1 = value; OnPropertyChanged(); } }
        #endregion
        #region M0
        private string _M0 = null;
        [Browsable(false)]
        public string M0 { get => _M0; set { _M0 = value; OnPropertyChanged(); } }
        #endregion
        #region M1
        private string _M1 = null;
        [Browsable(false)]
        public string M1 { get => _M1; set { _M1 = value; OnPropertyChanged(); } }
        #endregion
        #region S0
        private string _S0 = null;
        [Browsable(false)]
        public string S0 { get => _S0; set { _S0 = value; OnPropertyChanged(); } }
        #endregion
        #region S1
        private string _S1 = null;
        [Browsable(false)]
        public string S1 { get => _S1; set { _S1 = value; OnPropertyChanged(); } }
        #endregion

        #endregion

        public UIElement RealContentLine => ContentLine?.CloneElement();
        public UIElement RealSeparator => Separator?.CloneElement();

        #region DP (for numeric palette control)

        #region AnimationTime (DP SHORT)
        [Category("Palette")]
        public double AnimationTime { get { return (double)GetValue(AnimationTimeProperty); } set { SetValue(AnimationTimeProperty, value); } }
        public static readonly DependencyProperty AnimationTimeProperty = DependencyProperty.Register("AnimationTime", typeof(double), typeof(PaletteTimeControl), new PropertyMetadata(300D));
        #endregion

        #region AdjustTop (DP SHORT)
        [Category("Palette")]
        public double AdjustTop { get { return (double)GetValue(AdjustTopProperty); } set { SetValue(AdjustTopProperty, value); } }
        public static readonly DependencyProperty AdjustTopProperty = DependencyProperty.Register("AdjustTop", typeof(double), typeof(PaletteTimeControl), new PropertyMetadata(0D));
        #endregion

        #region ShowLine (DP SHORT)
        [Category("Palette")]
        public bool ShowLine { get { return (bool)GetValue(ShowLineProperty); } set { SetValue(ShowLineProperty, value); } }
        public static readonly DependencyProperty ShowLineProperty = DependencyProperty.Register("ShowLine", typeof(bool), typeof(PaletteTimeControl), new PropertyMetadata(true));
        #endregion

        #region LineColor (DP SHORT)
        [Category("Palette")]
        public Brush LineColor { get { return (Brush)GetValue(LineColorProperty); } set { SetValue(LineColorProperty, value); } }
        public static readonly DependencyProperty LineColorProperty = DependencyProperty.Register("LineColor", typeof(Brush), typeof(PaletteTimeControl), new PropertyMetadata(Brushes.White));
        #endregion

        #region ContentLine (DP SHORT)
        [Browsable(false)]
        public UIElement ContentLine { get { return (UIElement)GetValue(ContentLineProperty); } set { SetValue(ContentLineProperty, value); } }
        public static readonly DependencyProperty ContentLineProperty = DependencyProperty.Register("ContentLine", typeof(UIElement), typeof(PaletteTimeControl), new PropertyMetadata(null));
        #endregion

        #region NumericBackground (DP SHORT)
        [Category("Palette")]
        public Brush NumericBackground { get { return (Brush)GetValue(NumericBackgroundProperty); } set { SetValue(NumericBackgroundProperty, value); } }
        public static readonly DependencyProperty NumericBackgroundProperty = DependencyProperty.Register("NumericBackground", typeof(Brush), typeof(PaletteTimeControl), new PropertyMetadata(Brushes.Black));
        #endregion

        #region NumericCornerRadius (DP SHORT)
        [Category("Palette")]
        public CornerRadius NumericCornerRadius { get { return (CornerRadius)GetValue(NumericCornerRadiusProperty); } set { SetValue(NumericCornerRadiusProperty, value); } }
        public static readonly DependencyProperty NumericCornerRadiusProperty = DependencyProperty.Register("NumericCornerRadius", typeof(CornerRadius), typeof(PaletteTimeControl), new PropertyMetadata(null));
        #endregion

        #region NumericBorderBrush (DP SHORT)
        [Category("Palette")]
        public Brush NumericBorderBrush { get { return (Brush)GetValue(NumericBorderBrushProperty); } set { SetValue(NumericBorderBrushProperty, value); } }
        public static readonly DependencyProperty NumericBorderBrushProperty = DependencyProperty.Register("NumericBorderBrush", typeof(Brush), typeof(PaletteTimeControl), new PropertyMetadata(null));
        #endregion

        #region NumericBorderThickness (DP SHORT)
        [Category("Palette")]
        public Thickness NumericBorderThickness { get { return (Thickness)GetValue(NumericBorderThicknessProperty); } set { SetValue(NumericBorderThicknessProperty, value); } }
        public static readonly DependencyProperty NumericBorderThicknessProperty = DependencyProperty.Register("NumericBorderThickness", typeof(Thickness), typeof(PaletteTimeControl), new PropertyMetadata(new Thickness()));
        #endregion

        #region NumericWidth (DP SHORT)
        [Category("Palette")]
        public double NumericWidth { get { return (double)GetValue(NumericWidthProperty); } set { SetValue(NumericWidthProperty, value); } }
        public static readonly DependencyProperty NumericWidthProperty = DependencyProperty.Register("NumericWidth", typeof(double), typeof(PaletteTimeControl), new PropertyMetadata(double.NaN));
        #endregion

        #region NumericHeight (DP SHORT)
        [Category("Palette")]
        public double NumericHeight { get { return (double)GetValue(NumericHeightProperty); } set { SetValue(NumericHeightProperty, value); } }
        public static readonly DependencyProperty NumericHeightProperty = DependencyProperty.Register("NumericHeight", typeof(double), typeof(PaletteTimeControl), new PropertyMetadata(double.NaN));
        #endregion

        #endregion

        #region DP (for control)

        #region NumberMode
        [Category("Palette")]
        public NumberMode NumberMode { get => (NumberMode)GetValue(NumberModeProperty); set => SetValue(NumberModeProperty, value); }
        public static readonly DependencyProperty NumberModeProperty =
            DependencyProperty.Register("NumberMode",
                                        typeof(NumberMode),
                                        typeof(PaletteTimeControl),
                                        new PropertyMetadata(NumberMode.One,
                                            (ss, ee) => (ss as PaletteTimeControl).UpdateTime(false)));
        #endregion

        #region ShowSecond
        [Category("Palette")]
        public bool ShowSecond { get => (bool)GetValue(ShowSecondProperty); set => SetValue(ShowSecondProperty, value); }
        public static readonly DependencyProperty ShowSecondProperty =
            DependencyProperty.Register("ShowSecond",
                                        typeof(bool),
                                        typeof(PaletteTimeControl),
                                        new PropertyMetadata(true));

        #endregion

        #region MarginNumber
        [Category("Palette")]
        public double MarginNumber { get => (double)GetValue(MarginNumberProperty); set => SetValue(MarginNumberProperty, value); }
        public static readonly DependencyProperty MarginNumberProperty =
            DependencyProperty.Register("MarginNumber",
                                        typeof(double),
                                        typeof(PaletteTimeControl),
                                        new PropertyMetadata(0D));
        #endregion

        #region CornerRadius (DP SHORT)
        [Category("Palette")]
        public CornerRadius CornerRadius { get { return (CornerRadius)GetValue(CornerRadiusProperty); } set { SetValue(CornerRadiusProperty, value); } }
        public static readonly DependencyProperty CornerRadiusProperty =
            DependencyProperty.Register("CornerRadius",
                typeof(CornerRadius), typeof(PaletteTimeControl),
                new PropertyMetadata(new CornerRadius(2.0)));
        #endregion

        #region Separator (DP SHORT)
        [Browsable(false)]
        public UIElement Separator { get { return (UIElement)GetValue(SeparatorProperty); } set { SetValue(SeparatorProperty, value); } }
        public static readonly DependencyProperty SeparatorProperty = DependencyProperty.Register(
            "Separator", typeof(UIElement), typeof(PaletteTimeControl),
            new FrameworkPropertyMetadata(null,
                (FrameworkPropertyMetadataOptions)0xff));
        #endregion

        #region SeparatorHour
        [Browsable(false)]
        public UIElement SeparatorHour { get => (UIElement)GetValue(SeparatorHourProperty); set => SetValue(SeparatorHourProperty, value); }
        public static readonly DependencyProperty SeparatorHourProperty =
            DependencyProperty.Register("SeparatorHour",
                                        typeof(UIElement),
                                        typeof(PaletteTimeControl),
                                        new PropertyMetadata(null));
        #endregion

        #region SeparatorSecond
        [Browsable(false)]
        public UIElement SeparatorSecond { get => (UIElement)GetValue(SeparatorSecondProperty); set => SetValue(SeparatorSecondProperty, value); }
        public static readonly DependencyProperty SeparatorSecondProperty =
            DependencyProperty.Register("SeparatorSecond",
                                        typeof(UIElement),
                                        typeof(PaletteTimeControl),
                                        new PropertyMetadata(null));
        #endregion

        #region IsPaused (DP SHORT)
        [Browsable(false)]
        public bool IsPaused { get => (bool)GetValue(IsPausedProperty); set => SetValue(IsPausedProperty, value); }
        public static readonly DependencyProperty IsPausedProperty =
            DependencyProperty.Register("IsPaused",
                                        typeof(bool),
                                        typeof(PaletteTimeControl),
                                        new PropertyMetadata(false));
        #endregion

        #endregion
    }

    public class NumberModeConverter : MarkupExtension, IValueConverter
    {
        public override object ProvideValue(IServiceProvider serviceProvider) => this;

        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return (NumberMode)value == NumberMode.One ? Visibility.Visible : Visibility.Collapsed;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
