﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Markup;
using System.Windows.Media;

namespace UltimateCorp.Palette
{
    public enum GeometryPosition
    {
        Top,
        Bottom,
    }
    public class Geometry : MarkupExtension
    {
        public GeometryPosition Position { get; set; } = GeometryPosition.Top;

        private static DependencyProperty ReferenceBindingSinkProperty =
            DependencyProperty.RegisterAttached("ReferenceBindingSink",
                typeof(object),
                typeof(Geometry),
                new FrameworkPropertyMetadata(null, FrameworkPropertyMetadataOptions.Inherits));

        public BindingBase Reference { get; set; }

        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            IProvideValueTarget target = serviceProvider.GetService(typeof(IProvideValueTarget)) as IProvideValueTarget;
            DependencyObject targetObject;
            DependencyProperty targetProperty;

            if (target != null && target.TargetObject is DependencyObject && target.TargetProperty is DependencyProperty)
            {
                targetObject = (DependencyObject)target.TargetObject;
                targetProperty = (DependencyProperty)target.TargetProperty;
            }
            else
            {
                return this; // magic
            }

            try
            {
                BindingOperations.SetBinding(targetObject,
                    Geometry.ReferenceBindingSinkProperty,
                    Reference);

                var referenceValue = targetObject.GetValue(ReferenceBindingSinkProperty);
                var value = referenceValue as FrameworkElement;

                value.SizeChanged += delegate
                {
                    var to = targetObject as UIElement;
                    to.Clip = GetGeometry(value);
                };
                return null; // GetGeometry(value);
            }
            catch
            {
                return null;
            }
        }

        protected virtual System.Windows.Media.Geometry GetGeometry(FrameworkElement element)
        {
            RectangleGeometry goe = new RectangleGeometry();
            if (Position == GeometryPosition.Top)
                goe.Rect = new Rect(0, 0, element.ActualWidth, element.ActualHeight / 2);
            else
                goe.Rect = new Rect(0, element.ActualHeight / 2, element.ActualWidth, element.ActualHeight / 2);
            return goe;
        }
    }

}
